################################################################################
#################### Script for creating Scorecard Type 3 ######################
############## (By Forecast month & Region and forecast month) #################
################################################################################

plevel = ""
table_column_header = "Target month"
scorecard_type <- "type-3"
scorecard_name <- "Target month by Region & Forecast month"


## Load metrics
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Visualization/SC_1_load_metrics.R")
loaded_metrics <- SC_load_metrics(recipe)

## Transform loaded metrics
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Visualization/SC_2_transform_loaded_metrics.R")
transformed_metrics <- transform_loaded_metrics(recipe, loaded_metrics)

mean.bias <- transformed_metrics$mean.bias
mean.corr <- transformed_metrics$mean.corr
mean.rpss <- transformed_metrics$mean.rpss
mean.crpss <- transformed_metrics$mean.crpss
mean.enssprerr <- transformed_metrics$mean.enssprerr

region.names <- transformed_metrics$region.names
region.names.figure <- transformed_metrics$region.names.figure
n.regions <- transformed_metrics$n.regions
n.start.months <- transformed_metrics$n.start.months
n.for.months <- transformed_metrics$n.for.months
n.dec <- transformed_metrics$n.dec


## Translate values by 1 to shift from start date to target month
mean.bias2 <- abind(mean.bias, mean.bias, mean.bias, along = 1)
for(ff in 2:n.for.months){
  mean.bias2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.bias2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
}
mean.bias2 <- mean.bias2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array

mean.corr2 <- abind(mean.corr, mean.corr, mean.corr, along = 1)
for(ff in 2:n.for.months){
  mean.corr2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.corr2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
}
mean.corr2 <- mean.corr2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array

mean.crpss2 <- abind(mean.crpss, mean.crpss, mean.crpss, along = 1)
for(ff in 2:n.for.months){
  mean.crpss2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.crpss2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
}
mean.crpss2 <- mean.crpss2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array

mean.rpss2 <- abind(mean.rpss, mean.rpss, mean.rpss, along = 1)
for(ff in 2:n.for.months){
  mean.rpss2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.rpss2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
}
mean.rpss2 <- mean.rpss2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array

mean.enssprerr2 <- abind(mean.enssprerr, mean.enssprerr, mean.enssprerr, along = 1)
for(ff in 2:n.for.months){
  mean.enssprerr2[(n.start.months + 1):(n.start.months * 2),,ff] <- mean.enssprerr2[(n.start.months + 2 - ff):(n.start.months * 2 + 1 - ff),,ff]
}
mean.enssprerr2 <- mean.enssprerr2[(n.start.months + 1):(n.start.months * 2),,] ## take only central part of the array



## Change names to correspond with names in function
mean.bias <- mean.bias2
mean.corr <- mean.corr2
mean.crpss <- mean.crpss2
mean.rpss <- mean.rpss2
mean.enssprerr <- mean.enssprerr2

rm(mean.bias2, mean.corr2, mean.crpss2, mean.rpss2, mean.enssprerr2)

currentworkingdir <- getwd()

## Create scorecard image
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Visualization/SC_3_create_scorecard.R")
scorecard_table <- SC_create_table(transformed_metrics)

## Save scorecard in desired directory
source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Visualization/SC_4_save_scorecard.R")
save_scorecard <- SC_save_image(system.filename, reference.filename, var.name, var.filename, start.year, end.year)

setwd(currentworkingdir)

