rm(list=ls())

##### TESTING DATA FILES #####

library(ncdf4)
library(s2dv)

## Verification suite data
#file <- nc_open("/esarchive/scratch/nmilders/scorecards_data/input_data/ecmwfs5/tas/scorecards_ecmwfs5_era5_tas_1993-2016_s01.nc")

## Previous scorecard data
file <- nc_open("/esarchive/scratch/nmilders/scorecards_data/EQCmetrics_some_examples/EQCmetrics-ecmwfs5-2m_temperature-1993to2016-s01.nc")


crps <- ncvar_get(file, "crps")
lats <- ncvar_get(file, 'latitude')
lons <- ncvar_get(file, 'longitude')

# Close netcdf file when finished
nc_close(file)

# Select desired latitude and longitude range
pos_lats <- which(lats > -30 & lats <= 30)
lats <- lats[pos_lats]
pos_lons <- which(lons >= 0 | lons <= 360)
lons <- lons[pos_lons]

# Define variable
dim(crps)

# Adjust variable to specified longitude and latitude range, and time period
# crpss <- crpss[pos_lons, pos_lats, 1:8] # lon, lat, member, ftime
# dim(crpss)
# names(dim(crpss)) <- c("longitude", "latitude", "ftime")


crps <- crps[1:6, pos_lons, pos_lats] # lon, lat, member, ftime
dim(crps)
names(dim(crps)) <- c("ftime", "longitude", "latitude")

#crps <- Reorder(crps, c("sdate","time", "lat", "lon"))


mean_crps_nc <- multiApply::Apply(data = crps, target_dims = c("longitude", "latitude"), fun = mean, na.rm = FALSE, ncores = 1)[[1]]



##### TESTING CRPS AND CRPSS FUNCTIONS ######



## Load libraries
library(startR)
library(s2dv)
#library(CSTools)
library(RColorBrewer)


################################ INPUT PARAMETERS ##############################

## Latitude and longitude
latmin = -30
latmax = 30
lonmin = 0
lonmax = 359.9

ini_year <- 1993
fin_year  <- 2016
years <- ini_year:fin_year
ftime <- 1   ## leadtime + 1
sday <- '01' ## start day
smonth <- '01' ## start month
exp_sdate <- paste0(smonth,sday)
obs_sdate <- '01' ## smonth + (ftime -1)

variable <- 'tas'

grid <- 'r360x180'
grid_method <- 'bilinear'

exp_path <- '/esarchive/exp/ecmwf/system5c3s/monthly_mean/$var$_f6h/$var$_$years$$sdate$.nc'

exp  <- startR::Start(dat = exp_path,
                      var = variable,
                      years = paste0(years),
                      sdate = exp_sdate,
                      ftime = indices(ftime),
                      lon = values(list(lonmin,lonmax)),
                      lat = values(list(latmin,latmax)),
                      lat_reorder = Sort(),
                      lon_reorder = CircularSort(-180,180),
                      member = 'all',
                      transform = CDORemapper,
                      transform_extra_cells = 2,
                      transform_params = list(grid = grid, method = grid_method),
                      transform_vars = c('lat','lon'),
                      synonims = list(lon = c('lon', 'longitude'),
                                      lat = c('lat', 'latitude'),
                                      member = c('member','ensemble'),
                                      ftime = c('ftime', 'time')),
                      return_vars = list(lat = NULL,
                                         lon = NULL,
                                         time = 'years'),
                      retrieve = TRUE)


## Load Observational data

obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$years$$sdate$.nc'

obs  <- startR::Start(dat = obs_path,
                      var = variable,
                      years = paste0(years),
                      sdate = obs_sdate,
                      ftime = 1,
                      lon = values(list(lonmin,lonmax)),
                      lat = values(list(latmin,latmax)),
                      lat_reorder = Sort(),
                      lon_reorder = CircularSort(-180,180),
                      transform = CDORemapper,
                      transform_extra_cells = 2,
                      transform_params = list(grid = grid, method = grid_method),
                      transform_vars = c('lat','lon'),
                      synonims = list(lon = c('lon', 'longitude'),
                                      lat = c('lat', 'latitude'),
                                      ftime = c('ftime', 'time')),
                      return_vars = list(lat = NULL,
                                         lon = NULL,
                                         time = 'years'),
                      retrieve = TRUE)


lat <- as.vector(attributes(obs)$Variables$common$lat)
lon <- as.vector(attributes(obs)$Variables$common$lon)


clim_obs <- multiApply::Apply(list(obs), target_dims = 'years', fun = mean)$output1
exp_ano <- multiApply::Apply(list(exp, clim_obs), target_dims = 'dat', fun = function(x, y) {
                                  x-y}, ncores = 6)$output1
obs_ano <- multiApply::Apply(list(obs, clim_obs), target_dims = NULL, fun = function(x, y) {
                                  x-y}, ncores = 6)$output1
crps_exp <- easyVerification::veriApply(verifun = 'EnsCrps', fcst = exp, obs = obs, tdim = which(names(dim(exp))=='sdate'), 
                                            ensdim = which(names(dim(exp))=='member'), na.rm = FALSE, ncpus = ncores)

cprs1 <- function(obs) {
  obs_time_len <- dim(obs)['years']
  ref <- array(data = rep(obs, each = obs_time_len), dim = c(obs_time_len, obs_time_len))
  names(dim(ref)) <- c('sdate', 'member')
# ref: [sdate, memb]; obs: [sdate]
  crps_ref <- .CRPS(exp = ref, obs = obs, time_dim = 'sdate', memb_dim = 'member',
                  dat_dim = NULL, Fair = FALSE)
}

crps_ref_ano <- multiApply::Apply(list(obs), target_dims = c('years'), fun = cprs1)$output1
crps_ref <- easyVerification::veriApply(verifun = 'EnsCrps', fcst = s2dv::InsertDim(s2dv::InsertDim(clim_obs, 1, 24, 'years'), posdim = 4, lendim = 25, name = 'member'),
                                        obs = obs, tdim = 1, ensdim = 4, na.rm = FALSE, ncpus = ncores)

  crpss <- 1 - mean(crps_exp) / mean(crps_ref)


crps <- drop(crps_metric)
names(dim(crps)) <- c("year", "lon", "lat")
mean_crps <- multiApply::Apply(data = crps, target_dims = c("year","lon", "lat"), fun = mean, 
                               na.rm = FALSE, ncores = 1)[[1]]




source('/esarchive/scratch/nmilders/gitlab/nadia/verification_metrics/CRPS.R')
crps_s2dv <- CRPS(drop(exp), drop(obs), time_dim = 'years', memb_dim = 'member', dat_dim = NULL, Fair = FALSE, ncores = NULL)


mean_crps_sd2v <- multiApply::Apply(data = crps_s2dv, target_dims = c("lon", "lat"), fun = mean, na.rm = FALSE, ncores = 1)[[1]]

obs <- drop(obs)
names(dim(obs)) <- c("year", "lon", "lat")
mean_obs = multiApply::Apply(data = obs, target_dims = c("lon", "lat"), fun = mean, na.rm = FALSE, ncores = 1)[[1]]

exp <- drop(exp)
names(dim(exp)) <- c("year", "lon", "lat", "member")
mean_exp = multiApply::Apply(data = exp, target_dims = c("lon", "lat" ), fun = mean, na.rm = FALSE, ncores = 1)[[1]]

crpss <- easyVerification::veriApply(verifun = 'EnsCrpss', fcst = mean_exp, obs = mean_obs, tdim = which(names(dim(exp))=='years'),
                                     ensdim = which(names(dim(exp))=='member'), na.rm = FALSE, ncpus = 1)






